@@ -1,11 +1,11 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2001 by all Contributors.
+  source code Copyright (c) 1996-2002 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 2.2 (the "License");
+  set forth in the SystemC Open Source License Version 2.3 (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
   License at http://www.systemc.org/. Software distributed by Contributors
@@ -33,7 +33,6 @@
 
  *****************************************************************************/
 
-
 #ifndef SC_FIXED_H
 #define SC_FIXED_H
 
@@ -41,6 +40,14 @@
 #include "systemc/datatypes/fx/sc_fix.h"
 
 
+namespace sc_dt
+{
+
+// classes defined in this module
+template <int W, int I, sc_q_mode Q, sc_o_mode O, int N> class sc_fixed;
+template <int W, int I, sc_q_mode Q, sc_o_mode O, int N> class sc_fixed_fast;
+
+
 // ----------------------------------------------------------------------------
 //  TEMPLATE CLASS : sc_fixed
 //
@@ -623,6 +630,8 @@ sc_fixed_fast<W,I,Q,O,N>::operator -- ()
     return *this;
 }
 
+} // namespace sc_dt
+
 
 #endif
 
