@@ -1,11 +1,11 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2001 by all Contributors.
+  source code Copyright (c) 1996-2002 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 2.2 (the "License");
+  set forth in the SystemC Open Source License Version 2.3 (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
   License at http://www.systemc.org/. Software distributed by Contributors
@@ -33,7 +33,6 @@
 
  *****************************************************************************/
 
-
 #ifndef SC_UFIX_H
 #define SC_UFIX_H
 
@@ -41,6 +40,11 @@
 #include "systemc/datatypes/fx/sc_fxnum.h"
 
 
+namespace sc_dt
+{
+
+// classes defined in this module
+class sc_ufix;
 class sc_ufix_fast;
 
 
@@ -1907,6 +1911,8 @@ sc_ufix_fast::operator -- ()
     return *this;
 }
 
+} // namespace sc_dt
+
 
 #endif
 
