@@ -1,11 +1,11 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2001 by all Contributors.
+  source code Copyright (c) 1996-2002 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 2.2 (the "License");
+  set forth in the SystemC Open Source License Version 2.3 (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
   License at http://www.systemc.org/. Software distributed by Contributors
@@ -40,10 +40,15 @@
 
  *****************************************************************************/
 
+
 #ifndef _32BIT_
 
-#include "systemc/datatypes/int/sc_int.h"
-#include "systemc/datatypes/int/sc_uint.h"
+#include "systemc/datatypes/int/sc_int_base.h"
+#include "systemc/datatypes/int/sc_uint_base.h"
+
+
+namespace sc_dt
+{
 
 #ifndef WIN32
 
@@ -4475,4 +4480,6 @@ const uint_type mask_int[SC_INTWIDTH][SC_INTWIDTH] =
 
 #endif // end of #ifndef WIN32
 
+} // namespace sc_dt
+
 #endif
